#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <morecolors>

#pragma newdecls required

//Booleans:
bool g_bLate; //Check for late loads (sm plugins load xxx)

bool g_bNoKill[MAXPLAYERS + 1] = {false, ...};
bool g_bTrusted[MAXPLAYERS + 1] = {false, ...};

//Strings:
char g_sClipboard[MAXPLAYERS + 1][64];
char g_PropsPath[PLATFORM_MAX_PATH];
char g_sSteamID[MAXPLAYERS + 1][32];
char g_UntrustedProps[PLATFORM_MAX_PATH];

//Console Variables:
ConVar g_cvNormalMax;
ConVar g_cvTrustedMax;

//Databases:
Database g_dbTrusted; //For the trusted list

//Numbers:
int g_iSpawned[MAXPLAYERS + 1] = {0, ...};
int g_iWeldEnt[MAXPLAYERS + 1] = {-1, ...}; //Weld stuff
int g_iWeldPhase[MAXPLAYERS + 1] = {0, ...}; //Weld stuff

//Key Values:
KeyValues g_kvPropNames;

//String Maps:
StringMap g_smPropOwners;

#define CHATTAG		"{green}[Phoneburnia]{default}"
#define CONTAG		"[Phoneburnia]"

public APLRes AskPluginLoad2(Handle hMyself, bool bLate, char[] sError, int iErr_max)
{
	g_bLate = bLate;
}

public void OnMapStart()
{
	PrecacheModel("sprites/laser.vmt");
}

public void OnPluginStart()
{
	if(g_bLate)
	{
		for(int i = 1; i <= MaxClients; i++)
		{
			if(IsClientAuthorized(i))
			{
				GetClientAuthId(i, AuthId_Steam2, g_sSteamID[i], sizeof(g_sSteamID[]));
				OnClientPutInServer(i);
			}
		}
	}
	
	Database.Connect(SQL_ConnectDatabase, "phoneburnia");

	g_smPropOwners = new StringMap();

	//Make the cvars:
	g_cvNormalMax = CreateConVar("phoneburnia_maxprops", "35", "Maximum props for untrusted");
	g_cvTrustedMax = CreateConVar("phoneburnia_maxtrustedprops", "400", "Maximum props for trusted");
	AutoExecConfig();

	BuildPath(Path_SM, g_UntrustedProps, sizeof(g_UntrustedProps), "data/phoneburnia/untrusted_props.ini");
	BuildPath(Path_SM, g_PropsPath, sizeof(g_PropsPath), "data/phoneburnia/prop_names.ini");
	g_kvPropNames = new KeyValues("Props");

	RegAdminCmd("sm_createentity", Command_CreateEntity, ADMFLAG_GENERIC, "Create an entity by classname");

	RegAdminCmd("sm_clean", Command_CleanMap, ADMFLAG_GENERIC, "Clean the map of all props");
	RegAdminCmd("sm_delall", Command_DeleteAll, ADMFLAG_GENERIC, "Delete all props spawned by a player");

	RegAdminCmd("sm_goto", Command_TeleToPlayer, ADMFLAG_GENERIC, "Teleport to a player");
	RegAdminCmd("sm_bring", Command_TelePlayerTo, ADMFLAG_GENERIC, "Teleport a player to you");
	RegAdminCmd("sm_teleport", Command_TelePlayerToPlayer, ADMFLAG_GENERIC, "Teleport a player to another player");

	RegAdminCmd("sm_airboat", Command_Airboat, ADMFLAG_GENERIC, "Spawn an airboat");

	RegAdminCmd("sm_addtrusted", Command_AddTrusted, ADMFLAG_GENERIC, "Add a player to the trusted list");
	RegAdminCmd("sm_removetrusted", Command_RemoveTrusted, ADMFLAG_GENERIC, "Remove a player from the trusted list");

	RegAdminCmd("sm_setowner", Command_SetOwner, ADMFLAG_GENERIC, "Set ownership of an entity");

	RegConsoleCmd("sm_freezeit", Command_FreezeIt, "Freeze a prop");
	RegConsoleCmd("sm_unfreezeit", Command_UnfreezeIt, "Unfreeze a prop");

	RegConsoleCmd("sm_noclip", Command_Noclip, "Toggle noclip");
	RegConsoleCmd("sm_nokill", Command_Nokill, "Toggle nokill");

	RegConsoleCmd("sm_copy", Command_Copy, "Copy a model name");
	RegConsoleCmd("sm_paste", Command_Paste, "Paste a model");

	RegConsoleCmd("sm_spawn", Command_Spawn, "Spawn a prop");
	RegConsoleCmd("sm_spawncount", Command_SpawnCount, "Check how many more props you can spawn");
	RegConsoleCmd("sm_dupe", Command_Duplicate, "Duplicate the targeted prop");

	RegConsoleCmd("sm_trail", Command_Trail, "Put a trail on something");

	RegConsoleCmd("sm_skin", Command_SetSkin, "Set the skin of a prop");
	RegConsoleCmd("sm_move", Command_Move, "Move a prop x/y/z");
	RegConsoleCmd("sm_rotate", Command_Rotate, "Rotate a prop x/y/z");
	RegConsoleCmd("sm_straighten", Command_Straighten, "Straighten a prop");
	RegConsoleCmd("sm_delete", Command_Delete, "Delete a prop");
	RegConsoleCmd("sm_collide", Command_Collide, "Make a prop have collision");
	RegConsoleCmd("sm_nocollide", Command_NoCollide, "Make a prop have collision");
	RegConsoleCmd("sm_weld", Command_Weld, "Weld/Parent a prop to another prop");
	RegConsoleCmd("sm_unweld", Command_UnWeld, "Unweld/Unparent a prop");
	RegConsoleCmd("sm_deweld", Command_UnWeld, "Unweld/Unparent a prop");

	RegConsoleCmd("sm_what", Command_What, "Check who owns a prop");
	RegConsoleCmd("sm_whom", Command_What, "Check who owns a prop");
	RegConsoleCmd("sm_propowner", Command_What, "Check who owns a prop");
	RegConsoleCmd("sm_owner", Command_What, "Check who owns a prop");

	RegConsoleCmd("sm_paint", Command_Paint, "Paint a prop");
}

public Action Command_Weld(int iClient, int iArgs)
{
	if(iClient == 0)
	{
		CReplyToCommand(iClient, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1 || !IsValidEdict(entity))
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		switch(g_iWeldPhase[client])
		{
			//1st use of command:
			case 0:
			{
				g_iWeldEnt[client] = EntIndexToEntRef(entity);
				g_iWeldPhase[client] = 1;

				CReplyToCommand(client, "%s Use !weld on the pro pyou wish to weld to.", CHATTAG);
				return Plugin_Handled;
			}

			//2nd use of command:
			case 1:
			{
				if(!IsValidEdict(g_iWeldEnt[client])) return Plugin_Handled;

				char tName[32]; Format(tName, sizeof(tName), "%i-%i", g_iWeldEnt[client], client);
				DispatchKeyValue(g_iWeldEnt[client], "targetname", tName);
				SetVariantString(tName);
				AcceptEntityInput(entity, "SetParent");

				g_iWeldPhase[client] = 0;
				g_iWeldEnt[client] = -1;

				CReplyToCommand(client, "%s Props welded.", CHATTAG);

				return Plugin_Handled;
			}
		}
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_UnWeld(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1 || !IsValidEdict(entity))
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		AcceptEntityInput(entity, "ClearParent");
		CReplyToCommand(client, "%s Entity has been unwelded.", CHATTAG);
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Copy(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char id[32], fmt[64], nickname[64];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));
	Format(fmt, sizeof(fmt), "%s-Name", id);
	g_smPropOwners.GetString(fmt, nickname, sizeof(nickname));
	strcopy(g_sClipboard[client], sizeof(g_sClipboard[]), nickname);

	CPrintToChat(client, "%s Model Name: {green}%s{default}.", CHATTAG, nickname);
	return Plugin_Handled;
}

public Action Command_Paste(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	ClientCommand(client, "sm_spawn %s", g_sClipboard[client]);
	return Plugin_Handled;
}

//This is ideally a recreation of impulse 83
//CBasePlayer::CreateAirboat
//See: https://github.com/ValveSoftware/source-sdk-2013/blob/0d8dceea4310fde5706b3ce1c70609d72a38efdf/sp/src/game/server/player.cpp#L6052
public Action Command_Airboat(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	int entity = CreateEntityByName("prop_vehicle_airboat");
	DispatchKeyValue(entity, "model", "models/airboat.mdl");
	DispatchKeyValue(entity, "solid", "6");
	DispatchKeyValue(entity, "vehiclescript", "scripts/vehicles/airboat.txt");
	DispatchSpawn(entity);
	ActivateEntity(entity);

	//Get eye endpoint position:
	float start[3], angle[3], end[3];
	GetClientEyePosition(client, start);
	GetClientEyeAngles(client, angle);

	TR_TraceRayFilter(start, angle, MASK_SOLID, RayType_Infinite, TraceEntityFilterPlayer, client);

	if(!TR_DidHit(INVALID_HANDLE)) return Plugin_Handled;
	TR_GetEndPosition(end, INVALID_HANDLE);



	TeleportEntity(entity, end, NULL_VECTOR, NULL_VECTOR);
	return Plugin_Handled;
}

public Action Command_Duplicate(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		if(g_bTrusted[client])
		{
			if(g_iSpawned[client] >= g_cvTrustedMax.IntValue)
			{
				CReplyToCommand(client, "%s You are at the spawn limit.", CHATTAG);
				return Plugin_Handled;
			}
		}
		else
		{
			if(g_iSpawned[client] >= g_cvNormalMax.IntValue)
			{
				CReplyToCommand(client, "%s You are at the spawn limit.", CHATTAG);
				return Plugin_Handled;
			}
		}

		float origin[3];
		GetEntPropVector(entity, Prop_Data, "m_vecOrigin", origin);
		origin[2] += 5.0;

		float angles[3];
		GetEntPropVector(entity, Prop_Data, "m_angRotation", angles);

		char nickname[32], fmt[64];
		Format(fmt, sizeof(fmt), "%s-Name", id);
		g_smPropOwners.GetString(fmt, nickname, sizeof(nickname));

		int collision = GetEntProp(entity, Prop_Send, "m_CollisionGroup");

		char model[128];
		GetEntPropString(entity, Prop_Data, "m_ModelName", model, sizeof(model));

		char class[32];
		GetEdictClassname(entity, class, sizeof(class));

		int edict = CreateEntityByName(class);

		SetEntityModel(edict, model);
		DispatchSpawn(edict);
		SetEntProp(edict, Prop_Send, "m_CollisionGroup", collision);

		IntToString(EntIndexToEntRef(edict), id, sizeof(id));
		g_smPropOwners.SetString(id, g_sSteamID[client]);

		Format(fmt, sizeof(fmt), "%s-Name", id);
		g_smPropOwners.SetString(fmt, nickname);

		TeleportEntity(edict, origin, angles, NULL_VECTOR);
		g_iSpawned[client]++;
		CReplyToCommand(client, "%s Entity duplicated.", CHATTAG);
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_SpawnCount(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	int left; 
	if(g_bTrusted[client]) left = g_cvTrustedMax.IntValue - g_iSpawned[client];
	else left = g_cvNormalMax.IntValue - g_iSpawned[client];

	CReplyToCommand(client, "%s You have {green}%i{default} spawns left.", CHATTAG, left);
	return Plugin_Handled;
}

public Action Command_RemoveTrusted(int client, int args)
{
	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_removetrusted <player>");
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	//Database isn't connected:
	if(g_dbTrusted == null)
	{
		CReplyToCommand(client, "%s An error has occurred, please try again later.", CHATTAG);
		return Plugin_Handled;
	}

	if(!g_bTrusted[player])
	{
		CReplyToCommand(client, "%s {green}%N{default} is not on the trusted list.", CHATTAG, player);
		return Plugin_Handled;
	}

	char query[256];
	Format(query, sizeof(query), "DELETE FROM phoneburnia_trusted WHERE steam_id = '%s';", g_sSteamID[player]);
	g_dbTrusted.Query(SQL_GenericQuery, query);
	g_bTrusted[player] = false;

	CPrintToChat(player, "%s You have been remove from the trusted list.", CHATTAG);
	CReplyToCommand(client, "%s You have removed {green}%N{default} from the trusted list.", CHATTAG, player);
	return Plugin_Handled;
}

public Action Command_AddTrusted(int client, int args)
{
	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_addtrusted <player>");
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	//Database isn't connected:
	if(g_dbTrusted == null)
	{
		CReplyToCommand(client, "%s An error has occurred, please try again later.", CHATTAG);
		return Plugin_Handled;
	}

	if(g_bTrusted[player])
	{
		CReplyToCommand(client, "%s {green}%N{default} is already on the trusted list.", CHATTAG, player);
		return Plugin_Handled;
	}

	char query[256];
	Format(query, sizeof(query), "INSERT INTO phoneburnia_trusted (steam_id) VALUES ('%s');", g_sSteamID[player]);
	g_dbTrusted.Query(SQL_GenericQuery, query);
	g_bTrusted[player] = true;

	CPrintToChat(player, "%s You have been added to the trusted list.", CHATTAG);
	CReplyToCommand(client, "%s You have added {green}%N{default} to the trusted list.", CHATTAG, player);
	return Plugin_Handled;
}

public void SQL_ConnectDatabase(Database db, const char[] error, any data)
{
	if(db == null)
	{
		LogError("SQLite Error: %s", error);
		return;
	}

	char create[256];
	Format(create, sizeof(create), "CREATE TABLE IF NOT EXISTS phoneburnia_trusted (userid INTEGER PRIMARY KEY AUTOINCREMENT, steam_id TEXT NOT NULL);")
	g_dbTrusted = db;
	g_dbTrusted.Query(SQL_GenericQuery, create);

	PrintToServer("[Phoneburnia] SQLite Database Connected!");

	if(g_bLate)
	{
		for(int i = 1; i <= MaxClients; i++)
		{
			if(IsClientInGame(i) && IsClientAuthorized(i))
			{
				char query[256];
				Format(query, sizeof(query), "SELECT * FROM phoneburnia_trusted WHERE steam_id = '%s';", g_sSteamID[i]);
				g_dbTrusted.Query(SQL_CheckTrusted, query, GetClientSerial(i));
			}
		}
	}
}

public void OnClientAuthorized(int client, const char[] auth)
{
	GetClientAuthId(client, AuthId_Steam2, g_sSteamID[client], sizeof(g_sSteamID[]));
}

public void OnClientPutInServer(int client)
{
	g_bNoKill[client] = false;
	g_bTrusted[client] = false;
	g_iSpawned[client] = 0;

	SDKHook(client, SDKHook_OnTakeDamage, OnTakeDamage);

	if(g_dbTrusted != null)
	{
		char query[256];
		Format(query, sizeof(query), "SELECT * FROM phoneburnia_trusted WHERE steam_id = '%s';", g_sSteamID[client]);
		g_dbTrusted.Query(SQL_CheckTrusted, query, GetClientSerial(client));
	}
}

public Action OnTakeDamage(int victim, int &attacker, int &inflictor, float &damage, int &damagetype, int &weapon, float damageForce[3], float damagePosition[3])
{
	if(victim > 0 && victim <= MaxClients && attacker > 0 && attacker <= MaxClients)
	{
		if(g_bNoKill[victim] || g_bNoKill[attacker])
		{
			damage = 0.0;
			return Plugin_Changed;
		}
	}

	return Plugin_Continue;
}

public void SQL_GenericQuery(Database db, DBResultSet results, const char[] error, any data)
{
	int client = GetClientFromSerial(data);
	if(client >= 0) return;

	if(db == null)
	{
		PrintToServer("SQLite Error: %s", error);
		return;
	}

	if(results == null)
	{
		PrintToServer("SQLite Error: %s", error);
		return;
	}
}

public void SQL_CheckTrusted(Database db, DBResultSet results, const char[] error, any data)
{
	int client = GetClientFromSerial(data);
	if(client <= 0) return;

	if(db == null)
	{
		PrintToServer("SQLite Error: %s", error);
		return;
	}

	if(results == null)
	{
		PrintToServer("SQLite Error: %s", error);
		return;
	}

	if(results.RowCount > 0)
	{
		g_bTrusted[client] = true;
		CPrintToChat(client, "%s Welcome, {green}%N{default}. You have {green}Trusted{default} status.", CHATTAG, client);
	}
}

public void OnClientDisconnect(int client)
{
	for(int i = MaxClients + 1; i < GetMaxEntities(); i++)
	{
		if(IsValidEdict(i))
		{
			char class[32];
			GetEdictClassname(i, class, sizeof(class));
			if(StrContains(class, "prop_", false) != -1)
			{
				char owner[32], id[32];
				IntToString(EntIndexToEntRef(i), id, sizeof(id));
				g_smPropOwners.GetString(id, owner, sizeof(owner));

				//Drop prop spawn count by 1:
				if(StrEqual(g_sSteamID[client], owner, false))
				{
					g_iSpawned[client]--;
					g_smPropOwners.Remove(id);
					AcceptEntityInput(i, "Kill");
				}
			}
		}
	}

	g_sSteamID[client] = "";
}

public Action Command_Straighten(int client, int args)
{
	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		float angles[3];
		GetEntPropVector(entity, Prop_Data, "m_angRotation", angles);

		angles[0] = 0.0;
		angles[1] = 0.0;
		angles[2] = 0.0;
		TeleportEntity(entity, NULL_VECTOR, angles, NULL_VECTOR);

		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Spawn(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s This command cannot be ran from server console", CHATTAG);
		return Plugin_Handled;
	}

	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_spawn <prop name>", CHATTAG);
		return Plugin_Handled;
	}

	if(g_bTrusted[client])
	{
		if(g_iSpawned[client] >= g_cvTrustedMax.IntValue)
		{
			return Plugin_Handled;
		}

		g_kvPropNames.ImportFromFile(g_PropsPath);
		g_kvPropNames.JumpToKey("Models", false);

		char arg[32];
		GetCmdArg(1, arg, sizeof(arg));

		char data[256];
		g_kvPropNames.GetString(arg, data, sizeof(data), "NULL");

		char stuff[2][128];
		ExplodeString(data, "^", stuff, 2, sizeof(stuff[]));

		char class[64]; strcopy(class, sizeof(class), stuff[0]);
		char model[128]; strcopy(model, sizeof(model), stuff[1]);

		if(StrEqual(model, "NULL", false))
		{
			CReplyToCommand(client, "%s Invalid Model Name: {green}%s{default}.", CHATTAG, arg);
			return Plugin_Handled;
		}

		int entity = CreateEntityByName(class);
		if(!IsModelPrecached(model)) PrecacheModel(model, true);
		SetEntityModel(entity, model);

		//Get eye endpoint position:
		float start[3], angle[3], end[3];
		GetClientEyePosition(client, start);
		GetClientEyeAngles(client, angle);

		TR_TraceRayFilter(start, angle, MASK_SOLID, RayType_Infinite, TraceEntityFilterPlayer, client);

		if(!TR_DidHit(INVALID_HANDLE)) return Plugin_Handled;
		TR_GetEndPosition(end, INVALID_HANDLE);

		TeleportEntity(entity, end, NULL_VECTOR, NULL_VECTOR);
		DispatchSpawn(entity);

		char id[32];
		IntToString(EntIndexToEntRef(entity), id, sizeof(id));
		g_smPropOwners.SetString(id, g_sSteamID[client]);

		char fmt[64];
		Format(fmt, sizeof(fmt), "%s-Name", id);
		g_smPropOwners.SetString(fmt, arg);

		g_kvPropNames.Rewind();
		g_iSpawned[client]++;
	}
	else
	{
		if(g_iSpawned[client] > g_cvNormalMax.IntValue)
		{
			return Plugin_Handled;
		}

		g_kvPropNames.ImportFromFile(g_UntrustedProps);
		g_kvPropNames.JumpToKey("Models", false);

		char arg[32];
		GetCmdArg(1, arg, sizeof(arg));

		char data[256];
		g_kvPropNames.GetString(arg, data, sizeof(data), "NULL");

		char stuff[2][128];
		ExplodeString(data, "^", stuff, 2, sizeof(stuff[]));

		char class[64]; strcopy(class, sizeof(class), stuff[0]);
		char model[128]; strcopy(model, sizeof(model), stuff[1]);

		if(StrEqual(model, "NULL", false))
		{
			CReplyToCommand(client, "%s Invalid Model Name: {green}%s{default}.", CHATTAG, arg);
			return Plugin_Handled;
		}

		int entity = CreateEntityByName(class);
		if(!IsModelPrecached(model)) PrecacheModel(model, true);
		SetEntityModel(entity, model);

		//Get eye endpoint position:
		float start[3], angle[3], end[3];
		GetClientEyePosition(client, start);
		GetClientEyeAngles(client, angle);

		TR_TraceRayFilter(start, angle, MASK_SOLID, RayType_Infinite, TraceEntityFilterPlayer, client);

		if(!TR_DidHit(INVALID_HANDLE)) return Plugin_Handled;
		TR_GetEndPosition(end, INVALID_HANDLE);

		TeleportEntity(entity, end, NULL_VECTOR, NULL_VECTOR);
		DispatchSpawn(entity);

		char id[32];
		IntToString(EntIndexToEntRef(entity), id, sizeof(id));
		g_smPropOwners.SetString(id, g_sSteamID[client]);

		char fmt[64];
		Format(fmt, sizeof(fmt), "%s-Name", id);
		g_smPropOwners.SetString(fmt, arg);

		g_kvPropNames.Rewind();
		g_iSpawned[client]++;
	}

	return Plugin_Handled;
}

public Action Command_NoCollide(int client, int args)
{
	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		SetEntProp(entity, Prop_Send, "m_CollisionGroup", 1);
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Collide(int client, int args)
{
	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		SetEntProp(entity, Prop_Send, "m_CollisionGroup", 8);
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Delete(int client, int args)
{
	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		g_smPropOwners.Remove(id);
		AcceptEntityInput(entity, "Kill");
		g_iSpawned[client]--;
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Rotate(int client, int args)
{
	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_rotate <x> <y> <z>", CHATTAG)
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		float angles[3];
		GetEntPropVector(entity, Prop_Data, "m_angRotation", angles);

		char arg[32], arg2[32], arg3[32];
		GetCmdArg(1, arg, sizeof(arg));
		GetCmdArg(2, arg2, sizeof(arg2));
		GetCmdArg(3, arg3, sizeof(arg3));

		float x = StringToFloat(arg);
		float y = StringToFloat(arg2);
		float z = StringToFloat(arg3);

		angles[0] += x;
		angles[1] += y;
		angles[2] += z;

		TeleportEntity(entity, NULL_VECTOR, angles, NULL_VECTOR);
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Move(int client, int args)
{
	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_move <x> <y> <z>", CHATTAG)
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		float origin[3];
		GetEntPropVector(entity, Prop_Data, "m_vecOrigin", origin);

		char arg[32], arg2[32], arg3[32];
		GetCmdArg(1, arg, sizeof(arg));
		GetCmdArg(2, arg2, sizeof(arg2));
		GetCmdArg(3, arg3, sizeof(arg3));

		float x = StringToFloat(arg);
		float y = StringToFloat(arg2);
		float z = StringToFloat(arg3);

		origin[0] += x;
		origin[1] += y;
		origin[2] += z;

		TeleportEntity(entity, origin, NULL_VECTOR, NULL_VECTOR);
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_SetSkin(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_skin <id>", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		char arg[32];
		GetCmdArg(1, arg, sizeof(arg));

		SetEntProp(entity, Prop_Send, "m_nSkin", StringToInt(arg));
		CReplyToCommand(client, "%s Set the skin of this entity to: {green}%i{default}.", CHATTAG, StringToInt(arg));
		return Plugin_Handled;
	}

	return Plugin_Handled;
}

public Action Command_Nokill(int client, int args)
{
	if(g_bNoKill[client])
	{
		CReplyToCommand(client, "%s You have {red}Disabled{default} Nokill.", CHATTAG);
		g_bNoKill[client] = false;
		return Plugin_Handled;
	}
	else
	{
		CReplyToCommand(client, "%s You have {green}Enabled{default} Nokill.", CHATTAG);
		g_bNoKill[client] = true;
		return Plugin_Handled;
	}
}

public Action Command_DeleteAll(int client, int args)
{
	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_delall <player>", CHATTAG);
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);

	//invalid:
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	int deleted = 0;

	for(int i = MaxClients + 1; i < GetMaxEntities(); i++)
	{
		if(IsValidEdict(i))
		{
			char class[32];
			GetEdictClassname(i, class, sizeof(class));
			if(StrContains(class, "prop_", false) != -1)
			{
				char owner[32], id[32];
				IntToString(EntIndexToEntRef(i), id, sizeof(id));
				g_smPropOwners.GetString(id, owner, sizeof(owner));

				//Drop prop spawn count by 1:
				if(StrEqual(g_sSteamID[player], owner, false))
				{
					g_iSpawned[player]--;
					g_smPropOwners.Remove(id);
					AcceptEntityInput(i, "Kill");
					deleted++;
				}
			}
		}
	}

	CReplyToCommand(client, "%s Deleted %i props spawned by: {green}%N{default}.", CHATTAG, deleted, player);
	return Plugin_Handled;
}

public Action Command_CleanMap(int client, int args)
{
	for(int i = MaxClients + 1; i < GetMaxEntities(); i++)
	{
		if(IsValidEdict(i))
		{
			char class[32];
			GetEdictClassname(i, class, sizeof(class));
			if(StrContains(class, "prop_", false) != -1)
			{
				char owner[32], id[32];
				IntToString(EntIndexToEntRef(i), id, sizeof(id));
				g_smPropOwners.GetString(id, owner, sizeof(owner));

				//Drop prop spawn count by 1:
				for(int x = 1; x < MaxClients; x++)
				{
					if(StrEqual(g_sSteamID[x], owner, false)) g_iSpawned[x]--;
				}

				g_smPropOwners.Remove(id);
				AcceptEntityInput(i, "Kill");
			}
		}
	}
}

public Action Command_Trail(int client, int args)
{
	if(client == 0)
	{
		CReplyToCommand(client, "%s You cannot use this command from the server console", CHATTAG);
		return Plugin_Handled;
	}

	bool useHex = false;
	char arg[32]; 

	if(args == 1) 
	{
		GetCmdArg(1, arg, sizeof(arg));
		useHex = true;
	}
	else if(args < 3 && args != 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_trail <r> <g> <b> OR sm_trail <#HEXCOL>", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1 || !IsValidEdict(entity))
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		if(useHex)
		{
			//# + RRGGBB:
			if(strlen(arg) == 7)
			{
				ReplaceString(arg, sizeof(arg), "#", "", false);
				int hex; 
				StringToIntEx(arg, hex, 16);
				int r = (hex & 0xFF0000) >> 16;
				int g = (hex & 0xFF00) >> 8;
				int b = (hex & 0xFF);

				char color[16]; Format(color, sizeof(color), "%i %i %i", r, g, b);

				int trail = CreateEntityByName("env_spritetrail");
				DispatchKeyValue(trail, "lifetime", "1.0");
				DispatchKeyValue(trail, "startwidth", "10.0");
				DispatchKeyValue(trail, "endwidth", "1.0");
				DispatchKeyValue(trail, "spritename", "materials/sprites/laser.vmt");
				DispatchKeyValue(trail, "renderamt", "255");
				DispatchKeyValue(trail, "rendercolor", color);
				DispatchKeyValue(trail, "rendermode", "5");
				DispatchSpawn(trail);

				float origin[3]; GetEntPropVector(entity, Prop_Data, "m_vecOrigin", origin);
				TeleportEntity(trail, origin, NULL_VECTOR, NULL_VECTOR);

				char tName[32]; Format(tName, sizeof(tName), "%i-%i", entity, trail);
				DispatchKeyValue(entity, "targetname", tName);
				SetVariantString(tName);
				AcceptEntityInput(trail, "SetParent");

				CReplyToCommand(client, "%s You have put a trail on this entity. Color: \x07%06X#%X{default}.", CHATTAG, hex, hex);
				return Plugin_Handled;
			}

			//Just RRGGBB:
			else if(strlen(arg) == 6)
			{
				ReplaceString(arg, sizeof(arg), "#", "", false);
				int hex; 
				StringToIntEx(arg, hex, 16);
				int r = (hex & 0xFF0000) >> 16;
				int g = (hex & 0xFF00) >> 8;
				int b = (hex & 0xFF);

				char color[16]; Format(color, sizeof(color), "%i %i %i", r, g, b);

				int trail = CreateEntityByName("env_spritetrail");
				DispatchKeyValue(trail, "lifetime", "1.0");
				DispatchKeyValue(trail, "startwidth", "10.0");
				DispatchKeyValue(trail, "endwidth", "1.0");
				DispatchKeyValue(trail, "spritename", "materials/sprites/laser.vmt");
				DispatchKeyValue(trail, "renderamt", "255");
				DispatchKeyValue(trail, "rendercolor", color);
				DispatchKeyValue(trail, "rendermode", "5");
				DispatchSpawn(trail);

				float origin[3]; GetEntPropVector(entity, Prop_Data, "m_vecOrigin", origin);
				TeleportEntity(trail, origin, NULL_VECTOR, NULL_VECTOR);

				char tName[32]; Format(tName, sizeof(tName), "%i-%i", entity, trail);
				DispatchKeyValue(entity, "targetname", tName);
				SetVariantString(tName);
				AcceptEntityInput(trail, "SetParent");

				CReplyToCommand(client, "%s You have put a trail on this entity. Color: \x07%06X#%X{default}.", CHATTAG, hex, hex);
				return Plugin_Handled;
			}
			//apparently not a hex:
			else
			{
				CReplyToCommand(client, "%s This is not a valid color.", CHATTAG);
				return Plugin_Handled;
			}
		}
		else
		{
			char r[8], g[8], b[8];
			GetCmdArg(1, r, sizeof(r));
			GetCmdArg(2, g, sizeof(g));
			GetCmdArg(3, b, sizeof(b));

			int red = StringToInt(r);
			int green = StringToInt(g);
			int blue = StringToInt(b);

			char color[16]; Format(color, sizeof(color), "%i %i %i", red, green, blue);

			int trail = CreateEntityByName("env_spritetrail");
			DispatchKeyValue(trail, "lifetime", "1.0");
			DispatchKeyValue(trail, "startwidth", "10.0");
			DispatchKeyValue(trail, "endwidth", "1.0");
			DispatchKeyValue(trail, "spritename", "materials/sprites/laser.vmt");
			DispatchKeyValue(trail, "renderamt", "255");
			DispatchKeyValue(trail, "rendercolor", color);
			DispatchKeyValue(trail, "rendermode", "5");
			DispatchSpawn(trail);
			SetEntityRenderColor(trail, red, green, blue, 255);

			float origin[3]; GetEntPropVector(entity, Prop_Data, "m_vecOrigin", origin);
			TeleportEntity(trail, origin, NULL_VECTOR, NULL_VECTOR);

			char tName[32]; Format(tName, sizeof(tName), "%i-%i", entity, trail);
			DispatchKeyValue(entity, "targetname", tName);
			SetVariantString(tName);
			AcceptEntityInput(trail, "SetParent");

			//Convert back to hexadecimal:
			int hex; 
			hex |= (red & 0xFF) << 16;
			hex |= (green & 0xFF) << 8;
			hex |= (blue & 0xFF);

			CReplyToCommand(client, "%s You have put a trail on this entity. Color: \x07%06X%i %i %i{default}.", CHATTAG, hex, red, green, blue);
			return Plugin_Handled;
		}
	}

	return Plugin_Handled;
}

public Action Command_Paint(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	//Hexadecimal support:
	bool useHex = false;
	char arg[32]; 

	if(args == 1) 
	{
		GetCmdArg(1, arg, sizeof(arg));
		useHex = true;
	}
	else if(args < 3 && args != 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_paint <r> <g> <b> OR sm_paint <#HEXCOL>", CHATTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		if(!StrEqual(g_sSteamID[client], owner, false))
		{
			CReplyToCommand(client, "%s You do not own this entity", CHATTAG);
			return Plugin_Handled;
		}

		if(useHex)
		{
			//# + RRGGBB:
			if(strlen(arg) == 7)
			{
				ReplaceString(arg, sizeof(arg), "#", "", false);
				int hex; 
				StringToIntEx(arg, hex, 16);
				int r = (hex & 0xFF0000) >> 16;
				int g = (hex & 0xFF00) >> 8;
				int b = (hex & 0xFF);

				SetEntityRenderColor(entity, r, g, b);

				CReplyToCommand(client, "%s You have set the color of this entity to: \x07%06X#%X{default}.", CHATTAG, hex, hex);
				return Plugin_Handled;
			}

			//Just RRGGBB:
			else if(strlen(arg) == 6)
			{
				ReplaceString(arg, sizeof(arg), "#", "", false);
				int hex; 
				StringToIntEx(arg, hex, 16);
				int r = (hex & 0xFF0000) >> 16;
				int g = (hex & 0xFF00) >> 8;
				int b = (hex & 0xFF);

				SetEntityRenderColor(entity, r, g, b);

				CReplyToCommand(client, "%s You have set the color of this entity to: \x07%06X#%X{default}.", CHATTAG, hex, hex);
				return Plugin_Handled;
			}
			//apparently not a hex:
			else
			{
				CReplyToCommand(client, "%s This is not a valid color.", CHATTAG);
				return Plugin_Handled;
			}
		}
		else
		{
			char r[8], g[8], b[8];
			GetCmdArg(1, r, sizeof(r));
			GetCmdArg(2, g, sizeof(g));
			GetCmdArg(3, b, sizeof(b));

			int red = StringToInt(r);
			int green = StringToInt(g);
			int blue = StringToInt(b);

			SetEntityRenderColor(entity, red, green, blue);

			//Convert back to hexadecimal:
			int hex; 
			hex |= (red & 0xFF) << 16;
			hex |= (green & 0xFF) << 8;
			hex |= (blue & 0xFF);

			CReplyToCommand(client, "%s You have set the color of this entity to: \x07%06X%i %i %i{default}.", CHATTAG, hex, red, green, blue);
			return Plugin_Handled;
		}
	}
	return Plugin_Handled;
}

public Action Command_What(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	bool found;
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		for(int i = 1; i <= MaxClients; i++)
		{
			if(StrEqual(g_sSteamID[i], owner, false))
			{
				found = true;
				CReplyToCommand(client, "%s The owner of this entity is: {green}%N{default}.", CHATTAG, i);
				break;
			}
		}

		//If it's not found, remove it from the owner list:
		if(!found)
		{
			g_smPropOwners.Remove(id);
			CReplyToCommand(client, "%s This entity does not have an owner.", CHATTAG);
		}

		return Plugin_Handled;
	}

	CReplyToCommand(client, "%s This entity does not have an owner.", CHATTAG);
	return Plugin_Handled;
}

public Action Command_SetOwner(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);

	//invalid:
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	IntToString(EntIndexToEntRef(entity), id, sizeof(id));

	if(g_smPropOwners.GetString(id, owner, sizeof(owner)))
	{
		g_smPropOwners.SetString(id, g_sSteamID[player], true);
		CReplyToCommand(client, "%s Set the owner of this entity to: {green}%N{default}.", CHATTAG, player);
		return Plugin_Handled;
	}

	g_smPropOwners.SetString(id, g_sSteamID[player], true);
	CReplyToCommand(client, "%s Set the owner of this entity to: {green}%N{default}.", CHATTAG, player);
	return Plugin_Handled;
}

public Action Command_Noclip(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	if(GetEntityMoveType(client) != MOVETYPE_NOCLIP)
	{
		SetEntityMoveType(client, MOVETYPE_NOCLIP);
		CReplyToCommand(client, "%s Noclip {green}Enabled{default}.", CHATTAG);
	}
	else
	{
		SetEntityMoveType(client, MOVETYPE_WALK);
		CReplyToCommand(client, "%s Noclip {red}Disabled{default}.", CHATTAG);
	}

	return Plugin_Handled;
}

public Action Command_FreezeIt(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity.", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	int ref = EntIndexToEntRef(entity);
	IntToString(ref, id, sizeof(id));

	g_smPropOwners.GetString(id, owner, sizeof(owner));
	if(!StrEqual(owner, g_sSteamID[client], false))
	{
		CReplyToCommand(client, "%s You do not own this prop.", CHATTAG);
		return Plugin_Handled;
	}

	AcceptEntityInput(entity, "DisableMotion");
	return Plugin_Handled;
}

public Action Command_UnfreezeIt(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	int entity = GetClientAimTarget(client, false);
	if(entity == -1)
	{
		CReplyToCommand(client, "%s Invalid Entity.", CHATTAG);
		return Plugin_Handled;
	}

	char owner[32], id[32];
	int ref = EntIndexToEntRef(entity);
	IntToString(ref, id, sizeof(id));

	g_smPropOwners.GetString(id, owner, sizeof(owner));
	if(!StrEqual(owner, g_sSteamID[client], false))
	{
		CReplyToCommand(client, "%s You do not own this prop.", CHATTAG);
		return Plugin_Handled;
	}

	AcceptEntityInput(entity, "EnableMotion");
	return Plugin_Handled;
}

public Action Command_TelePlayerToPlayer(int client, int args)
{
	if(args < 2)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_teleport <player> <other player>", CHATTAG);
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);

	//invalid:
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	char arg2[32];
	GetCmdArg(2, arg2, sizeof(arg2));

	int otherPlayer = FindTarget(client, arg2, true, false);

	//invalid:
	if(otherPlayer == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg2);
		return Plugin_Handled;
	}

	float origin[3];
	GetClientAbsOrigin(player, origin);

	origin[2] += 65;
	TeleportEntity(otherPlayer, origin, NULL_VECTOR, NULL_VECTOR);
	return Plugin_Handled;
}

public Action Command_TeleToPlayer(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_goto <player>", CHATTAG);
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);

	//invalid:
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	float origin[3];
	GetClientAbsOrigin(player, origin);

	origin[2] += 65;
	TeleportEntity(client, origin, NULL_VECTOR, NULL_VECTOR);
	return Plugin_Handled;
}

public Action Command_TelePlayerTo(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_bring <player>", CHATTAG);
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int player = FindTarget(client, arg, true, false);

	//invalid:
	if(player == -1)
	{
		CReplyToCommand(client, "%s Invalid Target: {green}%s{default}.", CHATTAG, arg);
		return Plugin_Handled;
	}

	//Get eye endpoint position:
	float start[3], angle[3], end[3];
	GetClientEyePosition(client, start);
	GetClientEyeAngles(client, angle);

	TR_TraceRayFilter(start, angle, MASK_SOLID, RayType_Infinite, TraceEntityFilterPlayer, client);

	if(!TR_DidHit(INVALID_HANDLE)) return Plugin_Handled;
	TR_GetEndPosition(end, INVALID_HANDLE);

	TeleportEntity(player, end, NULL_VECTOR, NULL_VECTOR);
	return Plugin_Handled;
}

public Action Command_CreateEntity(int client, int args)
{
	if(client == 0)
	{
		PrintToServer("%s This command cannot be executed from the server console.", CONTAG);
		return Plugin_Handled;
	}

	if(args < 1)
	{
		CReplyToCommand(client, "%s Invalid Syntax: sm_createentity <classname>", CHATTAG);
		return Plugin_Handled;
	}

	char arg[32];
	GetCmdArg(1, arg, sizeof(arg));

	int entity = CreateEntityByName(arg);

	if(entity == -1)
	{
		CReplyToCommand(client, "%s Failed to create entity of classname: %s", CHATTAG, arg);
		return Plugin_Handled;
	}

	//Get eye endpoint position:
	float start[3], angle[3], end[3];
	GetClientEyePosition(client, start);
	GetClientEyeAngles(client, angle);

	TR_TraceRayFilter(start, angle, MASK_SOLID, RayType_Infinite, TraceEntityFilterPlayer, client);

	if(!TR_DidHit(INVALID_HANDLE)) return Plugin_Handled;
	TR_GetEndPosition(end, INVALID_HANDLE);

	TeleportEntity(entity, end, NULL_VECTOR, NULL_VECTOR);
	DispatchSpawn(entity);

	CReplyToCommand(client, "%s Created entity: {green}%s{default}.", CHATTAG, arg);
	return Plugin_Handled;
}

public bool TraceEntityFilterPlayer(int entity, int contentsMask, any data) 
{
	return entity > MaxClients;
}  

public Plugin myinfo = 
{
	name = "Phoneburnia Redux",
	author = "sidezz",
	description = "A recreation of an old build plugin",
	version = "1.0.1",
	url = ""
};